ENVIRONMENTS = [
    "Development",
    "Production",
    "Learn",
    "Staging",  
    ]

APP_ENVIRONMENT_SETTINGS_KEY = "APP_ENVIRONMENT_SETTINGS"

APP_ENVIRONMENT_KEY = "APP_ENVIRONMENT"

SETTINGS_JSON_PATH = "settings.json"

EVENTHUB_CONNECTION_STRING_KEY = "ConnectionString"

EVENTHUB_NAME_KEY = "Name"





import argparse, JsonParser, traceback
from Settings.Configuration import Configuration
from EventHub.Response import Response
from EventHub.EventHubProducer import EventHubProducer

def run():
    events_json = JsonParser.parse(args.events)
    event_type = args.type
    client_code = args.code
    account_name = args.account
    env_name = args.env
    return send_events(event_type, env_name, client_code, account_name, events_json)

def send_events(event_type, env_name, client_code, account_name, events):
    try:
        configuration = Configuration()
        configuration.init(env_name)

        producer = EventHubProducer(configuration.get_connection_string(), configuration.get_eventhub_name())

        return send_batch_with_partition_key(event_type, event_type, client_code, account_name, events, producer)
    except Exception as eh_err:
        return Response(False, traceback.format_exc()).toJSON()  

def send_batch_with_partition_key(event_type, env_name, client_code, account_name, events, producer):
    try:
        if not isinstance(producer, EventHubProducer):
            raise Exception("producer is not valid.")

        if not event_type or event_type is None:
            raise Exception("event_type is not provided.")

        if not env_name or env_name is None:
            raise Exception("env_name is not provided.")

        if not account_name or account_name is None:
            raise Exception("account_name is not provided.")

        if not client_code or client_code is None:
            raise Exception("account_name is not provided.")

        if not isinstance(events, list):
            raise Exception("events is not valid.")

        producer.send_batch_with_partition_key(event_type, event_type, client_code, account_name, events)

        return Response(True, "").toJSON()
    except Exception as eh_err:
        return Response(False, traceback.format_exc()).toJSON()  

#Declare script parameters
parser = argparse.ArgumentParser()
parser.add_argument("--env", help="Sets environment. Development, Production, QA or Learn", type=str)
parser.add_argument("--code", help="Sets Storis client code", type=str)
parser.add_argument("--events", help="Sets Storis client code")
parser.add_argument("--account", help="Sets Storis Account name")
parser.add_argument("--type", help="Sets Storis Event type")

args, unknown = parser.parse_known_args()


    



import sys, os, JsonParser, SupportedEvents, traceback
from os.path import dirname, abspath
from EventHub.Response import Response

sys.path.append(os.path.join(os.path.dirname(__file__)))
sys.path.append(dirname(dirname(abspath(__file__))))

import EventHub.SendEvents as SendEvents

def validate_event_type(event_type):
    if not event_type in SupportedEvents.EVENTS:
        raise Exception(event_type + " is not valid event type. Valid events are: " + JsonParser.serialize(SupportedEvents.EVENTS))

def run(eventData):
    try:
        event = JsonParser.parse(eventData)
        env_name = JsonParser.get_by_key(event, "environment")
        event_type = JsonParser.get_by_key(event, "eventType")
        client_code = JsonParser.get_by_key(event, "clientCode")
        account_name = JsonParser.get_by_key(event, "accountName")
        events_data = JsonParser.get_by_key(event, "data")

        #validate event type
        validate_event_type(event_type)

        return SendEvents.send_events(event_type, env_name, client_code, account_name, events_data)
    except Exception as eh_err:
        return Response(False, traceback.format_exc()).toJSON()

## for testing 
if sys.stdin and len(sys.argv) > 1:
    print(SendEvents.run())
    print('Events were sent.')





import JsonParser

class Settings(object):
    def __init__(self, connectionString, name):
        if not connectionString or connectionString is None:
            raise Exception("connectionString is required. Check settings.json file.")
        if not name or name is None:
            raise Exception("connectionString is name. Check settings.json file.")

        self.connectionString = connectionString
        self.name = name

    def object_decoder(obj):
        if '__type__' in obj and obj['__type__'] == 'Settings':
            return Settings(
                JsonParser.get_by_key(obj, 'connectionString'),
                JsonParser.get_by_key(obj, 'name'))
        return obj



ORDER_WRITTEN_SALE = "OrderWrittenSale"
ORDER_WRITTEN_LAYAWAY = "OrderWrittenLayaway"
ORDER_WRITTEN_QUICK_SALE = "OrderWrittenQuickSale"
ORDER_WRITTEN_QUOTE = "OrderWrittenQuote"

ORDER_DELETED_SALE = "OrderDeletedSale"

FULFILMENT_COMPLETED_CUSTOMER_PICKUP = "FulfillmentCompletedCustomerPickup"
FULFILMENT_COMPLETED_DELIVERY = "FulfillmentCompletedDelivery"
FULFILMENT_COMPLETED_DIRECTSHIP = "FulfillmentCompletedDirectShip"
FULFILMENT_COMPLETED_TAKEWITH = "FulfillmentCompletedTakeWith"

FULFILMENT_SCHEDULED_CUSTOMER_PICKUPY = "FulfillmentScheduledCustomerPickup"
FULFILMENT_SCHEDULED_DELIVERY = "FulfillmentScheduledDelivery"

EVENTS = [
        ORDER_WRITTEN_SALE,
        ORDER_WRITTEN_LAYAWAY,
        ORDER_WRITTEN_QUICK_SALE,
        ORDER_WRITTEN_QUOTE,
        ORDER_DELETED_SALE,    
        FULFILMENT_COMPLETED_CUSTOMER_PICKUP,    
        FULFILMENT_COMPLETED_DELIVERY,    
        FULFILMENT_COMPLETED_DIRECTSHIP,    
        FULFILMENT_COMPLETED_TAKEWITH,    
        FULFILMENT_SCHEDULED_CUSTOMER_PICKUPY,    
        FULFILMENT_SCHEDULED_DELIVERY,    
    ]



import os, JsonParser
from os.path import dirname, abspath
import Settings.SettingsDefaults as SettingsDefaults

class Configuration:
    def init(self, environment_name):
        if not environment_name in SettingsDefaults.ENVIRONMENTS:
            raise Exception(environment_name + " is not valid environment. Valid events are: " + JsonParser.serialize(SettingsDefaults.ENVIRONMENTS))
        if not os.getenv(SettingsDefaults.APP_ENVIRONMENT_KEY) is environment_name:
            os.environ[SettingsDefaults.APP_ENVIRONMENT_KEY] = environment_name
            self.read_settings()

    def get_environment(self):
        env = os.getenv(SettingsDefaults.APP_ENVIRONMENT_KEY)
        if not env or env is None:
            raise Exception("Environment is not set.")
        return env

    def get_environment_settings(self):
        settingsJson = os.getenv(SettingsDefaults.APP_ENVIRONMENT_SETTINGS_KEY)
        if not settingsJson or settingsJson is None:
            raise Exception("Environment settings are not set")
        settings = JsonParser.parse(settingsJson)

        return JsonParser.get_by_key(settings, self.get_environment())

    def read_settings(self):
        directoiry = dirname(dirname(dirname(abspath(__file__))))
        settings_file_path = os.path.join(directoiry, SettingsDefaults.SETTINGS_JSON_PATH)
        settings = JsonParser.parse_from_file(settings_file_path)
        self.set_environment_settings(settings)

    def set_environment_settings(self, settings):
        os.environ[SettingsDefaults.APP_ENVIRONMENT_SETTINGS_KEY] = JsonParser.serialize(settings)
        
    def get_connection_string(self):
        settings = self.get_environment_settings()
        return JsonParser.get_by_key(settings, SettingsDefaults.EVENTHUB_CONNECTION_STRING_KEY)

    def get_eventhub_name(self):
        settings = self.get_environment_settings()
        return JsonParser.get_by_key(settings, SettingsDefaults.EVENTHUB_NAME_KEY)



import asyncio
from azure.eventhub import EventData
from azure.eventhub.aio import EventHubProducerClient

class EventHubProducer:
    def __init__(self, connection_string, eventhub_name):
        if not connection_string or connection_string is None:
            raise Exception("Event HUB connection string is not configured. Make sure it is added to settings.json")
        if not eventhub_name or eventhub_name is None:
            raise Exception("Event HUB name is not configured. Make sure it is added to settings.json")

        self.connection_string = connection_string
        self.eventhub_name = eventhub_name

    def create_event_data(self, event_body, client_code, account_name, event_type):
        event_data = EventData(str(event_body))
        event_data.properties = {'event_type': event_type, 'client_code': client_code, 'account_name': account_name}
        return event_data

    async def create_batch_with_partition_key(self, partition_key, event_type, client_code, account_name, events):
        #Create batch with partition key
        event_data_batch_with_partition_key = await self.producer.create_batch(partition_key=partition_key)

        #Add events to the batch.
        length = len(events)
        for index in range(0, length):
            event_data = self.create_event_data(events[index], client_code, account_name, event_type)
            event_data_batch_with_partition_key.add(event_data)

        return event_data_batch_with_partition_key

    def send_batch_with_partition_key(self, partition_key, event_type, client_code, account_name, events):  
        self.producer = EventHubProducerClient.from_connection_string(
            conn_str=self.connection_string, 
            eventhub_name=self.eventhub_name)

        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        event_data_batch_with_partition_key = loop.run_until_complete(self.create_batch_with_partition_key(partition_key, event_type, client_code, account_name, events))
        event_hub_send_batch_result = loop.run_until_complete(self.producer.send_batch(event_data_batch_with_partition_key))
        loop.close()
        
        return event_hub_send_batch_result



import json

def parse(json_string):
    if not json_string or json_string is None:
        raise Exception("json_string is required.")
    return json.loads(json_string)

def get_by_key(data, key):
    if key in data:
        return data[key]
    else:
        raise Exception(key + " does not exists.")

def parse_from_file(path):
    if not path or path is None:
        raise Exception("File path is required.")
    jsonFile = open(path, "r")
    result = json.loads(jsonFile.read())
    jsonFile.close()
    return result

def serialize(obj):
    return json.dumps(obj)